@@ -200,6 +200,7 @@ class Source(DataStream):
             self.put(EnumeratedItem(i, item))
 
 
+
 class Computation(DataStream):
 
     def __init__(self, f, inputs, run_local=False, debug=False):
@@ -237,13 +238,6 @@ class Computation(DataStream):
         Engine.run_async(self.f, args, forward_result, self.run_local, self.debug)
 
 
-"""
-todo: cleanup size computation so it is only triggered when computation starts
-      more tests using a "delayed" Source
-      separate wf setup and running in envipy
-"""
-
-
 class OutputNode(DataStream):
 
     def __init__(self, input_):
@@ -390,6 +384,23 @@ class FullStreamComputation(SingleInputComputation):
         Engine.run_async(self.f, (values,), forward_result, self.run_local, self.debug)
 
 
+class ComputedSource(Computation):
+
+    def __init__(self, f, args):
+        super(ComputedSource, self).__init__(f, [], True, False)
+        self.f = f
+        self.args = args
+        self.items = None
+
+    def size(self):
+        return len(self.items)
+
+    def _start_computation(self):
+        self.items = list(self.f(*self.args))
+        for i, item in enumerate(self.items):
+            self.put(EnumeratedItem(i, item))
+
+
 class ZipComputation(Computation):
 
     """either input_streams provide all n inputs or one fixed input. The latter is managed by
@@ -506,8 +517,16 @@ fullstream_local = create_apply_decorator(FullStreamComputation, False, True)
 fullstream_debug = create_apply_decorator(FullStreamComputation, True, False)
 
 
+def source(inner):
+    @functools.wraps(inner)
+    def wrapped(*args):
+        return ComputedSource(inner, args)
+    wrapped.inner = inner
+    return wrapped
+
+
 def output(inner):
-    if isinstance(inner, Computation):
+    if isinstance(inner, (Computation)):
         return OutputNode(inner)
 
     @functools.wraps(inner)
